@@ -2,14 +2,13 @@ import numpy as np
 from matplotlib import pyplot as plt
 import pickle
 import random
-import EMpy
+import EMpy_gpu
 from emepy import tools
-from EMpy.modesolvers.FD import stretchmesh
+from scipy.signal import convolve2d
 
 
 class Mode(object):
-    """Object that holds the field profiles and effective index for an eigenmode
-    """
+    """Object that holds the field profiles and effective index for an eigenmode"""
 
     def __init__(self, x, y, wl, neff, Hx, Hy, Hz, Ex, Ey, Ez, n=None, width=None, thickness=None):
         """Constructor for Mode Object
@@ -71,50 +70,70 @@ class Mode(object):
             )
             self.n = eps_func(self.x, self.y)
 
-    def plot(self, operation="Real", colorbar=True):
+    def plot(self, operation="Real", colorbar=True, normalize=True):
         """Plots the fields in the mode using pyplot. Should call plt.figure() before and plt.show() or plt.savefig() after
 
         Parameters
         ----------
-        operation : string
-            the operation to perform on the fields from ("Real", "Imaginary", "Abs", "Abs^2") (default:"Real")
+        operation : string or function
+            the operation to perform on the fields from ("Real", "Imaginary", "Abs", "Abs^2") (default:"Real") or a function such as np.abs
         colorbar : bool
             if true, will show a colorbar for each field
+        normalize : bool
+            if true, will normalize biggest field to 1
         """
 
-        self /= max([np.abs(np.real(np.amax(i))) for i in [self.Ex, self.Ey, self.Ez, self.Hx, self.Hy, self.Hz]])
+        temp = (
+            self / max([np.abs(np.real(np.amax(i))) for i in [self.Ex, self.Ey, self.Ez, self.Hx, self.Hy, self.Hz]])
+            if normalize
+            else self / 1
+        )
 
         if operation == "Imaginary":
-            Hx = np.imag(self.Hx).T
-            Hy = np.imag(self.Hy).T
-            Hz = np.imag(self.Hz).T
-            Ex = np.imag(self.Ex).T
-            Ey = np.imag(self.Ey).T
-            Ez = np.imag(self.Ez).T
+            Hx = np.imag(temp.Hx).T
+            Hy = np.imag(temp.Hy).T
+            Hz = np.imag(temp.Hz).T
+            Ex = np.imag(temp.Ex).T
+            Ey = np.imag(temp.Ey).T
+            Ez = np.imag(temp.Ez).T
         elif operation == "Abs":
-            Hx = np.abs(self.Hx).T
-            Hy = np.abs(self.Hy).T
-            Hz = np.abs(self.Hz).T
-            Ex = np.abs(self.Ex).T
-            Ey = np.abs(self.Ey).T
-            Ez = np.abs(self.Ez).T
+            Hx = np.abs(temp.Hx).T
+            Hy = np.abs(temp.Hy).T
+            Hz = np.abs(temp.Hz).T
+            Ex = np.abs(temp.Ex).T
+            Ey = np.abs(temp.Ey).T
+            Ez = np.abs(temp.Ez).T
         elif operation == "Abs^2":
-            Hx = np.abs(self.Hx).T ** 2
-            Hy = np.abs(self.Hy).T ** 2
-            Hz = np.abs(self.Hz).T ** 2
-            Ex = np.abs(self.Ex).T ** 2
-            Ey = np.abs(self.Ey).T ** 2
-            Ez = np.abs(self.Ez).T ** 2
+            Hx = np.abs(temp.Hx).T ** 2
+            Hy = np.abs(temp.Hy).T ** 2
+            Hz = np.abs(temp.Hz).T ** 2
+            Ex = np.abs(temp.Ex).T ** 2
+            Ey = np.abs(temp.Ey).T ** 2
+            Ez = np.abs(temp.Ez).T ** 2
         elif operation == "Real":
-            Hx = np.real(self.Hx).T
-            Hy = np.real(self.Hy).T
-            Hz = np.real(self.Hz).T
-            Ex = np.real(self.Ex).T
-            Ey = np.real(self.Ey).T
-            Ez = np.real(self.Ez).T
+            Hx = np.real(temp.Hx).T
+            Hy = np.real(temp.Hy).T
+            Hz = np.real(temp.Hz).T
+            Ex = np.real(temp.Ex).T
+            Ey = np.real(temp.Ey).T
+            Ez = np.real(temp.Ez).T
         else:
-            raise Exception("Invalid operation entered. Please choose from ('Imaginary', 'Abs', 'Abs^2', 'Real')")
+            try:
+                Hx = operation(temp.Hx).T
+                Hy = operation(temp.Hy).T
+                Hz = operation(temp.Hz).T
+                Ex = operation(temp.Ex).T
+                Ey = operation(temp.Ey).T
+                Ez = operation(temp.Ez).T
+            except:
+                raise Exception(
+                    "Invalid operation provided. Please choose from ('Imaginary', 'Abs', 'Abs^2', 'Real') or provide a function"
+                )
 
+        if hasattr(operation, "__name__"):
+            operation = operation.__name__
+        else:
+            operation = str(operation)
         plt.subplot(2, 3, 4, adjustable="box", aspect=Ex.shape[0] / Ex.shape[1])
         v = max(abs(Ex.min()), abs(Ex.max()))
         plt.imshow(
@@ -122,7 +141,7 @@ class Mode(object):
             cmap="RdBu",
             vmin=-v,
             vmax=v,
-            extent=[self.x[0] * 1e6, self.x[-1] * 1e6, self.y[0] * 1e6, self.y[-1] * 1e6],
+            extent=[temp.x[0] * 1e6, temp.x[-1] * 1e6, temp.y[0] * 1e6, temp.y[-1] * 1e6],
         )
         plt.title(operation + "(Ex)")
         if colorbar:
@@ -137,7 +156,7 @@ class Mode(object):
             cmap="RdBu",
             vmin=-v,
             vmax=v,
-            extent=[self.x[0] * 1e6, self.x[-1] * 1e6, self.y[0] * 1e6, self.y[-1] * 1e6],
+            extent=[temp.x[0] * 1e6, temp.x[-1] * 1e6, temp.y[0] * 1e6, temp.y[-1] * 1e6],
         )
         plt.title(operation + "(Ey)")
         if colorbar:
@@ -152,7 +171,7 @@ class Mode(object):
             cmap="RdBu",
             vmin=-v,
             vmax=v,
-            extent=[self.x[0] * 1e6, self.x[-1] * 1e6, self.y[0] * 1e6, self.y[-1] * 1e6],
+            extent=[temp.x[0] * 1e6, temp.x[-1] * 1e6, temp.y[0] * 1e6, temp.y[-1] * 1e6],
         )
         plt.title(operation + "(Ez)")
         if colorbar:
@@ -167,7 +186,7 @@ class Mode(object):
             cmap="RdBu",
             vmin=-v,
             vmax=v,
-            extent=[self.x[0] * 1e6, self.x[-1] * 1e6, self.y[0] * 1e6, self.y[-1] * 1e6],
+            extent=[temp.x[0] * 1e6, temp.x[-1] * 1e6, temp.y[0] * 1e6, temp.y[-1] * 1e6],
         )
         plt.title(operation + "(Hx)")
         if colorbar:
@@ -182,7 +201,7 @@ class Mode(object):
             cmap="RdBu",
             vmin=-v,
             vmax=v,
-            extent=[self.x[0] * 1e6, self.x[-1] * 1e6, self.y[0] * 1e6, self.y[-1] * 1e6],
+            extent=[temp.x[0] * 1e6, temp.x[-1] * 1e6, temp.y[0] * 1e6, temp.y[-1] * 1e6],
         )
         plt.title(operation + "(Hy)")
         if colorbar:
@@ -197,7 +216,7 @@ class Mode(object):
             cmap="RdBu",
             vmin=-v,
             vmax=v,
-            extent=[self.x[0] * 1e6, self.x[-1] * 1e6, self.y[0] * 1e6, self.y[-1] * 1e6],
+            extent=[temp.x[0] * 1e6, temp.x[-1] * 1e6, temp.y[0] * 1e6, temp.y[-1] * 1e6],
         )
         plt.title(operation + "(Hz)")
         if colorbar:
@@ -207,40 +226,34 @@ class Mode(object):
 
         plt.tight_layout()
 
-    def _inner_product(self, mode1, mode2):
+    def _inner_product(self, mode1, mode2, mask=None):
         """Helper function that takes the inner product between Modes mode1 and mode2
 
         Parameters
         ----------
         mode1 : Mode
-            first eigenmode in the operation 
+            first eigenmode in the operation
         mode2 : Mode
             second eigenmode in the operation
 
         Returns
         -------
-        number 
+        number
             the inner product between the two input modes
         """
 
-        res = mode1.Ex.shape[0] * mode1.Ex.shape[1]
-        Ex = mode1.Ex.reshape(1, res)
-        Ey = mode1.Ey.reshape(1, res)
-        Ez = mode1.Ez.reshape(1, res)
-        L = np.stack([Ex, Ey, Ez], axis=-1)[0]
+        mask = 1 if mask is None else mask
 
-        res = mode2.Hx.shape[0] * mode2.Hx.shape[1]
-        Hx = np.conj(mode2.Hx).reshape(1, res)
-        Hy = np.conj(mode2.Hy).reshape(1, res)
-        Hz = np.conj(mode2.Hz).reshape(1, res)
-        R = np.stack([Hx, Hy, Hz], axis=-1)[0]
+        Ex = mode1.Ex * mask
+        Hy = np.conj(mode2.Hy) * mask
+        Ey = mode1.Ey * mask
+        Hx = np.conj(mode2.Hx) * mask
 
-        cross = np.cross(L, R)[:, 2]
-        size = int(np.sqrt(cross.shape[0]))
-        cross = cross.reshape((size, size))
+        cross = Ex * Hy - Ey * Hx
 
-        # cross = mode1.Ex * np.conj(mode2.Hy) - mode1.Ey * np.conj(mode2.Hx)
-        return np.trapz(np.trapz(cross, mode1.x), mode1.y)  # /np.trapz(np.trapz(cross, mode1.x), mode1.y) ### HEY
+        return np.trapz(
+            np.trapz(cross, np.real(mode1.x)), np.real(mode1.y)
+        )  # /np.trapz(np.trapz(cross, mode1.x), mode1.y) ### HEY
 
     def inner_product(self, mode2):
         """Takes the inner product between self and the provided Mode
@@ -252,12 +265,57 @@ class Mode(object):
 
         Returns
         -------
-        number 
+        number
             the inner product between the modes
         """
 
         return self._inner_product(self, mode2)  # / self._inner_product(self, self)
 
+    def check_spurious(self, threshold_power=0.05, threshold_neff=0.9):
+        """Takes in a mode and determine whether the mode is likely spurious based on the ratio of confined to not confined power
+
+        Parameters
+        ----------
+        threshold_power : float
+            threshold of power percentage of Pz in the core to total
+        threshold_neff : float
+            threshold of real to abs neff percentage
+
+        Returns 
+        -------
+        boolean
+            True if likely spurious 
+        """
+
+        power_bool = self.get_confined_power() < threshold_power
+        neff_bool = (np.real(self.neff) / np.abs(self.neff)) < threshold_neff
+        return power_bool or neff_bool
+
+    def get_confined_power(self, num_pixels=None):
+        """Takes in a mode and returns the percentage of power confined in the core
+
+        Parameters
+        ----------
+        num_pixels : int
+            number of pixels outside of the core to expand the mask to capture power just outside the core
+
+        Returns
+        -------
+        float
+            Percentage of confined power
+        """
+
+        # Increase core by 5% to capture slight leaks
+        if num_pixels is None:
+            num_pixels = int(len(self.x) * 0.05)
+
+        mask = np.where(self.n > np.mean(self.n), 1, 0)
+        kernel = np.ones((num_pixels + 1, num_pixels + 1))
+        mask = convolve2d(mask, kernel, "same")
+        mask = np.where(mask > 0, 1, 0)
+        ratio = self._inner_product(self, self, mask=mask) / self._inner_product(self, self, mask=None)
+        return ratio
+
     def __str__(self):
         return "Mode Object with effective index of " + str(self.neff)
 
@@ -336,16 +394,14 @@ class Mode(object):
         return self
 
     def normalize(self):
-        """Normalizes the Mode to power 1.
-        """
+        """Normalizes the Mode to power 1."""
         self.zero_phase()
         factor = self.inner_product(self)
         self /= np.sqrt(factor)
         return
 
     def zero_phase(self):
-        """Changes the phase such that the z components are all imaginary and the xy components are all real.
-        """
+        """Changes the phase such that the z components are all imaginary and the xy components are all real."""
 
         index = int(self.Hy.shape[0] / 2)
         phase = np.angle(np.array(self.Hy))[index][index]
@@ -354,32 +410,27 @@ class Mode(object):
             self *= -1
 
     def get_fields(self):
-        """Returns an array [self.Hx, self.Hy, self.Hz, self.Ex, self.Ey, self.Ez].
-        """
+        """Returns an array [self.Hx, self.Hy, self.Hz, self.Ex, self.Ey, self.Ez]."""
 
         return [self.Hx, self.Hy, self.Hz, self.Ex, self.Ey, self.Ez]
 
     def get_H(self):
-        """Returns an array [self.Hx, self.Hy, self.Hz].
-        """
+        """Returns an array [self.Hx, self.Hy, self.Hz]."""
 
         return [self.Hx, self.Hy, self.Hz]
 
     def get_E(self):
-        """Returns an array [self.Ex, self.Ey, self.Ez].
-        """
+        """Returns an array [self.Ex, self.Ey, self.Ez]."""
 
         return [self.Ex, self.Ey, self.Ez]
 
     def get_neff(self):
-        """Returns the effective index as a complex number.
-        """
+        """Returns the effective index as a complex number."""
 
         return self.neff
 
     def get_wavelength(self):
-        """Returns the wavelength.
-        """
+        """Returns the wavelength."""
 
         return self.wl
 
@@ -388,7 +439,7 @@ class Mode(object):
 
         Parameters
         ----------
-        path : string 
+        path : string
             The path (including name) to save the file.
         """
 
@@ -412,8 +463,6 @@ class Mode(object):
 
         from scipy.sparse import coo_matrix
 
-        core_index = tools.Si(self.wl * 1e6)
-        cladding_index = tools.SiO2(self.wl * 1e6)
         if self.n is None:
             self.epsfunc = tools.get_epsfunc(
                 self.width,
@@ -888,7 +937,7 @@ class Mode(object):
                         0.5
                         * ezz4
                         * (
-                            -k ** 2 * exy1
+                            -(k ** 2) * exy1
                             - (1 - exx1 / ezz1) / n / w
                             - exy1 / ezz1 * (-2.0 / n ** 2 - 2 / n ** 2 * (n - s) / s)
                         )
@@ -899,7 +948,7 @@ class Mode(object):
                         + 0.5
                         * ezz1
                         * (
-                            -k ** 2 * exy4
+                            -(k ** 2) * exy4
                             + (1 - exx4 / ezz4) / n / e
                             - exy4 / ezz4 * (-2.0 / n ** 2 - 2 / n ** 2 * (n - s) / s)
                         )
@@ -912,7 +961,7 @@ class Mode(object):
                         0.5
                         * ezz3
                         * (
-                            -k ** 2 * exy2
+                            -(k ** 2) * exy2
                             + (1 - exx2 / ezz2) / s / w
                             - exy2 / ezz2 * (-2.0 / s ** 2 + 2 / s ** 2 * (n - s) / n)
                         )
@@ -923,7 +972,7 @@ class Mode(object):
                         + 0.5
                         * ezz2
                         * (
-                            -k ** 2 * exy3
+                            -(k ** 2) * exy3
                             - (1 - exx3 / ezz3) / s / e
                             - exy3 / ezz3 * (-2.0 / s ** 2 + 2 / s ** 2 * (n - s) / n)
                         )
@@ -1307,7 +1356,7 @@ class Mode(object):
                         * ezz2
                         / eyy1
                         * (
-                            -k ** 2 * eyx1
+                            -(k ** 2) * eyx1
                             - (1 - eyy1 / ezz1) / n / w
                             - eyx1 / ezz1 * (-2.0 / w ** 2 + 2 / w ** 2 * (e - w) / e)
                         )
@@ -1318,7 +1367,7 @@ class Mode(object):
                         * ezz1
                         / eyy2
                         * (
-                            -k ** 2 * eyx2
+                            -(k ** 2) * eyx2
                             + (1 - eyy2 / ezz2) / s / w
                             - eyx2 / ezz2 * (-2.0 / w ** 2 + 2 / w ** 2 * (e - w) / e)
                         )
@@ -1331,7 +1380,7 @@ class Mode(object):
                         * ezz3
                         / eyy4
                         * (
-                            -k ** 2 * eyx4
+                            -(k ** 2) * eyx4
                             + (1 - eyy4 / ezz4) / n / e
                             - eyx4 / ezz4 * (-2.0 / e ** 2 - 2 / e ** 2 * (e - w) / w)
                         )
@@ -1341,7 +1390,7 @@ class Mode(object):
                         * ezz4
                         / eyy3
                         * (
-                            -k ** 2 * eyx3
+                            -(k ** 2) * eyx3
                             - (1 - eyy3 / ezz3) / s / e
                             - eyx3 / ezz3 * (-2.0 / e ** 2 - 2 / e ** 2 * (e - w) / w)
                         )
@@ -1551,10 +1600,10 @@ class Mode(object):
             v = n.reshape(nx, ny)[:-1, :-1]
 
             # in xc e yc
-            Dx = neff * EMpy.utils.centered2d(Hy) + (Hz[:-1, 1:] + Hz[1:, 1:] - Hz[:-1, :-1] - Hz[1:, :-1]) / (
+            Dx = neff * EMpy_gpu.utils.centered2d(Hy) + (Hz[:-1, 1:] + Hz[1:, 1:] - Hz[:-1, :-1] - Hz[1:, :-1]) / (
                 2j * k * v
             )
-            Dy = -neff * EMpy.utils.centered2d(Hx) - (Hz[1:, :-1] + Hz[1:, 1:] - Hz[:-1, 1:] - Hz[:-1, :-1]) / (
+            Dy = -neff * EMpy_gpu.utils.centered2d(Hx) - (Hz[1:, :-1] + Hz[1:, 1:] - Hz[:-1, 1:] - Hz[:-1, :-1]) / (
                 2j * k * h
             )
             Dz = (
@@ -1571,14 +1620,20 @@ class Mode(object):
             Eys.append(Ey)
             Ezs.append(Ez)
 
-        self.Hx = (Hx[1:, 1:] + Hx[1:, :-1] + Hx[:-1, 1:] + Hx[:-1, :-1]) / 4.0 + 0j
-        self.Hy = (Hy[1:, 1:] + Hy[1:, :-1] + Hy[:-1, 1:] + Hy[:-1, :-1]) / 4.0 + 0j
-        self.Hz = (Hzs[0][1:, 1:] + Hzs[0][1:, :-1] + Hzs[0][:-1, 1:] + Hzs[0][:-1, :-1]) / 4.0 + 0j
+        # self.Hx = (Hx[1:, 1:] + Hx[1:, :-1] + Hx[:-1, 1:] + Hx[:-1, :-1]) / 4.0 + 0j
+        # self.Hy = (Hy[1:, 1:] + Hy[1:, :-1] + Hy[:-1, 1:] + Hy[:-1, :-1]) / 4.0 + 0j
+        # self.Hz = (Hzs[0][1:, 1:] + Hzs[0][1:, :-1] + Hzs[0][:-1, 1:] + Hzs[0][:-1, :-1]) / 4.0 + 0j
+        self.Hx = Hxs[0]
+        self.Hy = Hys[0]
+        self.Hz = Hzs[0]
         self.Ex = Exs[0]
         self.Ey = Eys[0]
         self.Ez = Ezs[0]
-        self.x = (self.x[1:] + self.x[:-1]) / 2.0
-        self.y = (self.y[1:] + self.y[:-1]) / 2.0
+        x_ = (self.x[1:] + self.x[:-1]) / 2.0
+        y_ = (self.y[1:] + self.y[:-1]) / 2.0
+        self.Ex = tools.interp(self.x, self.y, x_, y_, Exs[0], False)
+        self.Ey = tools.interp(self.x, self.y, x_, y_, Eys[0], False)
+        self.Ez = tools.interp(self.x, self.y, x_, y_, Ezs[0], False)
         self.x = self.x - self.x[int(len(self.x) / 2)]
         self.y = self.y - self.y[int(len(self.y) / 2)]
         self.H = np.sqrt(np.abs(self.Hx) ** 2 + np.abs(self.Hy) ** 2 + np.abs(self.Hz) ** 2)
@@ -1598,8 +1653,7 @@ class Mode(object):
         tmp = self.epsfunc(xc, yc)
 
         def _reshape(tmp):
-            """pads the array by duplicating edge values
-            """
+            """pads the array by duplicating edge values"""
             tmp = np.c_[tmp[:, 0:1], tmp, tmp[:, -1:]]
             tmp = np.r_[tmp[0:1, :], tmp, tmp[-1:, :]]
             return tmp
@@ -1621,4 +1675,3 @@ class Mode(object):
             raise ValueError("Invalid eps")
 
         return epsxx, epsxy, epsyx, epsyy, epszz
-
